@@ -35,9 +35,6 @@
 #if HAVE_SYS_SOCKET_H
 #include <sys/socket.h>
 #endif
-#if HAVE_DMALLOC_H
-#include <dmalloc.h>
-#endif
 
 #include <net-snmp/net-snmp-includes.h>
 #include <net-snmp/agent/net-snmp-agent-includes.h>
@@ -136,14 +133,6 @@ close_agentx_session(netsnmp_session * session, int sessid)
 
     DEBUGMSGTL(("agentx/master", "close %08p, %d\n", session, sessid));
     if (sessid == -1) {
-        unregister_mibs_by_session(session);
-        unregister_index_by_session(session);
-        snmp_call_callbacks(SNMP_CALLBACK_APPLICATION,
-                            SNMPD_CALLBACK_REQ_UNREG_SYSOR_SESS,
-                            (void*)session);
-	if (session->myvoid != NULL) {
-	  free(session->myvoid);
-	}
         /*
          * The following is necessary to avoid locking up the agent when
          * a sugagent dies during a set request. We must clean up the
@@ -158,6 +147,12 @@ close_agentx_session(netsnmp_session * session, int sessid)
             }
         }
                 
+        unregister_mibs_by_session(session);
+        unregister_index_by_session(session);
+        snmp_call_callbacks(SNMP_CALLBACK_APPLICATION,
+                            SNMPD_CALLBACK_REQ_UNREG_SYSOR_SESS,
+                            (void*)session);
+	SNMP_FREE(session->myvoid);
         return AGENTX_ERR_NOERROR;
     }
 
@@ -393,10 +388,12 @@ add_agent_caps_list(netsnmp_session * session, netsnmp_pdu *pdu)
 
     parms.OR_oid = pdu->variables->name;
     parms.OR_oidlen = pdu->variables->name_length;
-    parms.OR_descr = (char *) pdu->variables->val.string;
+    parms.OR_descr  = netsnmp_strdup_and_null(pdu->variables->val.string,
+                                              pdu->variables->val_len);
     parms.OR_sess = sp;
     snmp_call_callbacks(SNMP_CALLBACK_APPLICATION,
                         SNMPD_CALLBACK_REQ_REG_SYSOR, (void*)&parms);
+    free(parms.OR_descr);
     return AGENTX_ERR_NOERROR;
 }
 
