@@ -4,6 +4,9 @@
  */
 
 #include <net-snmp/net-snmp-config.h>
+#include <net-snmp/net-snmp-includes.h>
+#include <net-snmp/agent/net-snmp-agent-includes.h>
+#include <net-snmp/agent/hardware/memory.h>
 #include "host_res.h"
 #include "hr_filesys.h"
 #include "hr_storage.h"
@@ -39,7 +42,7 @@
 #include <stdlib.h>
 #endif
 
-#if defined(aix4) || defined(aix5)
+#if defined(aix4) || defined(aix5) || defined(aix6)
 #include <sys/mntctl.h>
 #include <sys/vmount.h>
 #include <sys/statfs.h>
@@ -128,7 +131,7 @@ static int		fscount;
 #define HRFS_statfs	win_statfs
 #define	HRFS_type	f_type
 
-#elif defined(aix4) || defined(aix5)
+#elif defined(aix4) || defined(aix5) || defined(aix6)
 
 struct vmount *aixmnt, *aixcurr;
 struct HRFS_entry {
@@ -154,6 +157,7 @@ struct mntent  *HRFS_entry;
 #define MNTTYPE_MSDOS	"msdos"
 #define MNTTYPE_FAT32	"vfat"
 #define MNTTYPE_NTFS	"ntfs"
+#define MNTTYPE_NFS4	"nfs4"
 #endif	/* linux */
 
 #endif
@@ -202,7 +206,8 @@ init_hr_filesys(void)
 {
     REGISTER_MIB("host/hr_filesys", hrfsys_variables, variable4,
                  hrfsys_variables_oid);
-#if defined(aix4) || defined(aix5) /* something leaks, make it idiot-safe */
+#if defined(aix4) || defined(aix5) || defined(aix6) 
+    /* something leaks, make it idiot-safe */
     aixmnt = NULL;
     aixcurr = NULL;
 #endif
@@ -396,7 +401,7 @@ var_hrfilesys(struct variable *vp,
             break;
 #endif
         }
-#elif defined(aix4) || defined(aix5)
+#elif defined(aix4) || defined(aix5) || defined(aix6)
         switch (HRFS_entry->HRFS_type) {
         case MNT_AIX:
         case MNT_JFS:
@@ -485,6 +490,10 @@ var_hrfilesys(struct variable *vp,
         else if (!strcmp(mnt_type, MNTTYPE_NFS3))
             fsys_type_id[fsys_type_len - 1] = 14;
 #endif
+#ifdef MNTTYPE_NFS4
+        else if (!strcmp(mnt_type, MNTTYPE_NFS4))
+            fsys_type_id[fsys_type_len - 1] = 14;
+#endif
 #ifdef MNTTYPE_MFS
         else if (!strcmp(mnt_type, MNTTYPE_MFS))
             fsys_type_id[fsys_type_len - 1] = 8;
@@ -516,7 +525,7 @@ var_hrfilesys(struct variable *vp,
         long_return = HRFS_entry->f_flags & MNT_RDONLY ? 2 : 1;
 #elif defined(cygwin)
         long_return = 1;
-#elif defined(aix4) || defined(aix5)
+#elif defined(aix4) || defined(aix5) || defined(aix6)
         long_return = (HRFS_entry->HRFS_flags & MNT_READONLY) == 0 ? 1 : 2;
 #else
 #if HAVE_HASMNTOPT
@@ -535,7 +544,7 @@ var_hrfilesys(struct variable *vp,
             long_return = 2;    /* others probably aren't */
         return (u_char *) & long_return;
     case HRFSYS_STOREIDX:
-        long_return = fsys_idx + HRS_TYPE_FIXED_MAX;
+        long_return = fsys_idx + NETSNMP_MEM_TYPE_MAX;
         return (u_char *) & long_return;
     case HRFSYS_FULLDUMP:
         return when_dumped(HRFS_entry->HRFS_name, FULL_DUMP, var_len);
@@ -579,7 +588,7 @@ Init_HR_FileSys(void)
     getfsstat(fsstats, fscount * sizeof(*fsstats), MNT_NOWAIT);
 #endif
     HRFS_index = 0;
-#elif defined(aix4) || defined(aix5)
+#elif defined(aix4) || defined(aix5) || defined(aix6)
     int ret;
     uint size;
     ret = 0;
@@ -607,6 +616,11 @@ Init_HR_FileSys(void)
     if (fp != NULL)
         fclose(fp);
     fp = fopen(ETC_MNTTAB, "r");
+    if (!fp) {
+      char tmpbuf[STRINGMAX];
+      snprintf( tmpbuf, sizeof(tmpbuf), "Can't open %s\n", ETC_MNTTAB );
+      config_perror(tmpbuf);
+    }
 #endif
 }
 
@@ -671,7 +685,7 @@ Get_Next_HR_FileSys(void)
         return -1;
     HRFS_entry = fsstats + HRFS_index;
     return ++HRFS_index;
-#elif defined(aix4) || defined(aix5)
+#elif defined(aix4) || defined(aix5) || defined(aix6)
     if(aixcurr == NULL) {
         if(aixmnt != NULL) free(aixmnt);
         aixmnt = NULL;
@@ -711,9 +725,13 @@ Get_Next_HR_FileSys(void)
         return -1;
 #endif                          /* solaris2 */
 
+    DEBUGMSGTL(("host/hr_filesys", "Get_Next_HRFS %s\n", HRFS_entry->HRFS_name));
+
     for (cpp = HRFS_ignores; *cpp != NULL; ++cpp)
-        if (!strcmp(HRFS_entry->HRFS_type, *cpp))
+        if (!strcmp(HRFS_entry->HRFS_type, *cpp)) {
+            DEBUGMSGTL(("host/hr_filesys", "Get_Next_HRFS: skipping %s (%s)\n", HRFS_entry->HRFS_type, cpp));
             return Get_Next_HR_FileSys();
+        }
 
     /*
      * Try and ensure that index values are persistent
@@ -743,7 +761,7 @@ Check_HR_FileSys_NFS (void)
 #if HAVE_GETFSSTAT && !defined(MFSNAMELEN)
     if ((HRFS_entry->HRFS_type == MOUNT_NFS) ||
         (HRFS_entry->HRFS_type == MOUNT_AFS))
-#elif defined(aix4) || defined(aix5)
+#elif defined(aix4) || defined(aix5) || defined(aix6)
     if(HRFS_entry->HRFS_type == MNT_NFS || HRFS_entry->HRFS_type == MNT_NFS3)
 #else /* HAVE_GETFSSTAT */
     if ( HRFS_entry->HRFS_type != NULL && (
@@ -755,6 +773,9 @@ Check_HR_FileSys_NFS (void)
 #if defined(MNTTYPE_NFS3)
 	!strcmp( HRFS_entry->HRFS_type, MNTTYPE_NFS3) ||
 #endif
+#if defined(MNTTYPE_NFS4)
+	!strcmp( HRFS_entry->HRFS_type, MNTTYPE_NFS4) ||
+#endif
 #if defined(MNTTYPE_SMBFS)
 	!strcmp( HRFS_entry->HRFS_type, MNTTYPE_SMBFS) ||
 #endif
@@ -764,6 +785,7 @@ Check_HR_FileSys_NFS (void)
 #if defined(MNTTYPE_AFP)
 	!strcmp( HRFS_entry->HRFS_type, MNTTYPE_AFP) ||
 #endif
+	!strcmp( HRFS_entry->HRFS_type, "cifs") ||
 	    /*
 	     * MVFS is Rational ClearCase's view file system
 	     * it is similiar to NFS file systems in that it is mounted
@@ -783,7 +805,7 @@ End_HR_FileSys(void)
     if (fsstats)
         free((char *) fsstats);
     fsstats = NULL;
-#elif defined(aix4) || defined(aix5)
+#elif defined(aix4) || defined(aix5) || defined(aix6)
     if(aixmnt != NULL) {
         free(aixmnt);
         aixmnt = NULL;
