@@ -1,7 +1,7 @@
 /*
  * route data access header
  *
- * $Id: route.h,v 1.2 2004/09/02 00:08:30 rstory Exp $
+ * $Id: route.h 13271 2005-10-28 15:29:23Z rstory $
  */
 /**---------------------------------------------------------------------*/
 /*
@@ -18,12 +18,14 @@
  *    be handled in the *_hpux.h header file.
  */
 config_require(ip-forward-mib/data_access/route_common)
+
 #if defined( linux )
-config_require(ip-forward-mib/data_access/route_linux);
+config_require(ip-forward-mib/data_access/route_linux)
+config_require(ip-forward-mib/data_access/route_ioctl)
 #else
 #error "the route data access library is not available for this platform."
 #endif
 
 /** need interface for ifIndex */
-config_require(if-mib/data_access/interface);
+config_require(if-mib/data_access/interface)
 
