@@ -1,6 +1,6 @@
 #include "netbsd.h"
 
-#define netbsd1 1               /* we're really close to this */
+#define netbsd1 netbsd1         /* we're really close to this */
 #define UVM
 
 #undef MBPOOL_SYMBOL
@@ -11,3 +11,9 @@
 #ifndef UINT32_MAX
 #define UINT32_MAX	(4294967295U)
 #endif
+
+/* define the extra mib modules that are supported */
+#define NETSNMP_INCLUDE_HOST_RESOURCES
+#ifdef NETSNMP_ENABLE_MFD_REWRITES
+#define NETSNMP_INCLUDE_IFTABLE_REWRITES
+#endif
